#!/bin/bash
#byLeal

function dohelp(){
        echo "+---------------------------------------------------+"
        echo "| Simple script to fix Active Directory DNS Records |"
        echo "|                              Implementing Samba 4 |"
        echo "|                                                   |"
        echo "|                    Copyright(c) 2013 Marcelo Leal |"
        echo "+---------------------------------------------------+"
        echo " Usage:"
        echo ""
        echo " \"-h\" for this help message"
        echo " \"-l\" list the DNS records from the file." 
        echo " \"-c <IP>\" checks Samba 4 DNS entries."
        echo " \"-f <IP>\" fixes Samba 4 DNS entries." 
        echo ""
        echo " The base file can be obtained from the Microsoft"
        echo " Windows Server being replaced:"
        echo " C:\Windows\System32\config\netlogon.dns"
        echo ""
        echo " Error Codes:     (0)OK," 
        echo "                  (1)Wrong Options," 
        echo "                  (2)At least one record error,"
        echo "                  (3)Could not open dns.txt file."
}

function dolist(){
	while IFS=" " read a b c d e f g h
	do
		# A Records...
		if [ $d == "A" ]; then
			echo "Record Type [  $d  ]: $a IP: $e"	
		fi
		# SRV Records...
		if [ $d == "SRV" ]; then
			echo "Record Type [ $d ]: $a Hostname: $h"	
		fi
		# CNAME Records...
		if [ $d == "CNAME" ]; then
			echo "Record Type [$d]: $a Hostname: $e"	
		fi
	done<./dns.txt
}

function docheck(){
	ipdns=$1
	while IFS=" " read a b c d e f g h
	do
		echo -n "Record Type [  $d  ]: $a Number of entries:"	
		# A Records...
		if [ $d == "A" ]; then
			howmany=`host -t $d $a | wc -l`;
			if [ $howmany -eq 1 ]; then
				echo -n " $howmany [OK], IP registered:"
				ipreg=`host -t $d $a | awk '{print $NF}'`; 
				if [ $ipreg == "$ipdns" ]; then
					echo " [OK]."
				else
					echo " $ipreg [ERROR!]"
				fi
			else
				echo " $howmany [ERROR!]"
			fi
				
		fi
		# SRV Records...
		if [ $d == "SRV" ]; then
			howmany=`host -t $d $a | wc -l`;
			if [ $howmany -eq 1 ]; then
				echo -n " $howmany [OK], Hostname registered:"
				namereg=`host -t $d $a | awk '{print $NF}' | awk -F. '{print $1}'`;
				if [ $namereg == "$hostname" ]; then
					echo " [OK]."
				else
					echo " $namereg [ERROR!]"
				fi
			else
				echo " $howmany [ERROR!]"
			fi
		fi
		# CNAME Records...
		if [ $d == "CNAME" ]; then
			# The CNAME record is removed in the book to show the user some examples...
			# echo "Record Type [$d]: $a Hostname: $e - REMOVED IN THE BOOK AS AN EXAMPLE FOR THE READER"	
			echo ""
		fi
	done<./dns.txt
}

function dofix(){
        ipdns=$1
        while IFS=" " read a b c d e f g h
        do
		shortname=`echo $a | awk -F. '{print tolower($1)}'`;
		lastdomain=`echo $a | awk -F. '{for (i=2;i<=NF;++i) printf("%s.", $i)}' | awk '{print tolower($0)}'`;
		# Just removing remaining "." (dots)...
		rn=`echo $lastdomain | sed -e 's/\.*$//g' | awk '{print tolower($0)}'`;
                echo -n "Record Type [ $d ]: $a Number of entries:"
                howmany=`host -t $d $a | wc -l`;

                if [ $howmany -eq 1 ]; then
                        echo -n " $howmany [OK], registered:"
			if [ $d == "A" ]; then
                        	ipreg=`host -t $d $a | awk '{print $NF}'`;
                        	if [ $ipreg == "$ipdns" ]; then
                                	echo " [OK]."
                        	else
                                	echo -n " $ipreg [ERROR!] Fixing...:"
                                	eval /usr/local/samba/bin/samba-tool dns update $ipdns $rn $shortname A $ipreg $ipdns --password \'$password\' >/dev/null 2>&1
                             		if [ $? -eq 0 ]; then
                                       		echo " [OK]."
                               		else
                                       		echo " [ERROR!]"
                               		fi
                        	fi
			fi 
			if [ $d == "SRV" ]; then
                                namereg=`host -t $d $a | awk '{print $NF}' | awk -F. '{print tolower($1)}'`;
                                if [ $namereg == "$hostname" ]; then
                                        echo " [OK]."
                                else
                                        echo -n " $namereg [ERROR!] Fixing...:"
                                	eval /usr/local/samba/bin/samba-tool dns update $ipdns $rn $shortname SRV $ipreg $ipdns --password \'$password\' >/dev/null 2>&1
                             		if [ $? -eq 0 ]; then
                                       		echo " [OK]."
                               		else
                                       		echo " [ERROR!]"
					fi
                               	fi
                        fi

                else
                        echo -n " $howmany [ERROR!] Fixing...:";
			if [ $d == "A" ]; then
				tf=$(tempfile) || exit
				control=0;
				host -t $d $a > $tf;
				while IFS=" " read w1 w2 w3 w4
				do
					sn=`echo $w1 | awk -F. '{print tolower($1)}'`;
					fullname=`echo $w1 | awk -F. '{for (i=2;i<=NF;++i) printf("%s.", $i)}' | awk '{print tolower($0)}'`;
					fn=`echo $w1 | sed -e 's/\.*$//g' | awk '{print tolower($0)}'`;
					if [ $w4 != "$ipdns" ]; then
						if [ $fn == $zone ]; then
                               				eval /usr/local/samba/bin/samba-tool dns delete $ipdns $fn @ A $w4 --password \'$password\' >/dev/null 2>&1
						else
                               				eval /usr/local/samba/bin/samba-tool dns delete $ipdns $fn $sn A $w4 --password \'$password\' >/dev/null 2>&1
						fi
                               			if [ $? -eq 0 ]; then
                                       			echo " [OK]."
                               			else
                                      			echo " [ERROR!]"
                               			fi
					else
						control=1;
					fi
				done<$tf
				if [ $control -eq 0 ]; then
                               		eval /usr/local/samba/bin/samba-tool dns add $ipdns $fullname @ A $w4 --password \'$password\' >/dev/null 2>&1
                               		if [ $? -eq 0 ]; then
                               			echo " [OK]."
                               		else
                               			echo " [ERROR!]"
                               		fi
				fi
                	fi
			if [ $d == "SRV" ]; then
				tf=$(tempfile) || exit
				control=0;
				host -t $d $a > $tf;
				while IFS=" " read y1 y2 y3 y4 y5 y6 y7 y8
				do
					sn=`echo $y1 | awk '{print tolower($0)}' | sed -e "s/\.$zone//g" | sed -e 's/\.*$//g'`;
					srvname=`echo $y8 | awk -F. '{print tolower($1)}'`;
					fullname=`echo $y8 | awk '{print tolower($0)}' | sed -e 's/\.*$//g'`;
					if [ $srvname != "$hostname" ]; then
						if [ `echo $sn | grep _msdcs` >/dev/null 2>&1 ]; then
							sns=`echo $y1 | awk '{print tolower($0)}' | sed -e "s/\._msdcs\.$zone//g" | sed -e 's/\.*$//g'`;
                               				eval /usr/local/samba/bin/samba-tool dns delete $ipdns _msdcs.$zone $sns SRV \"$fullname $y7 $y5 $y6\" --password \'$password\' >/dev/null 2>&1
						else
                               				eval /usr/local/samba/bin/samba-tool dns delete $ipdns $zone $sn SRV \"$fullname $y7 $y5 $y6\" --password \'$password\' >/dev/null 2>&1
						fi
                               			if [ $? -eq 0 ]; then
                                       			echo " [OK]."
                               			else
                                      			echo " [ERROR!]."
                               			fi
					else
						control=1;
					fi
				done<$tf
				if [ $control -eq 0 ]; then
					if [ $fn == $zone ]; then
                               			eval /usr/local/samba/bin/samba-tool dns add $ipdns $zone $sn SRV \"$fullname $y7 $y5 $y6\" --password \'$password\'
					else
                               			eval /usr/local/samba/bin/samba-tool dns add $ipdns $zone $sn SRV \"$fullname $y7 $y5 $y6\" --password \'$password\'
					fi
                               		if [ $? -eq 0 ]; then
                               			echo " [OK]."
                               		else
                               			echo " [ERROR!]."
                               		fi
				fi
                	fi
		fi

                # CNAME Records...
                if [ $d == "CNAME" ]; then
			# The CNAME record is removed in the book to show the user some examples...
                        # echo "Record Type [$d]: $a Hostname: $e"
			echo ""
                fi
        done<./dns.txt
}

#Change to the values for your site...
#-------------------------------------
zone="poa.msdcbrz.eall.com.br";
password='w1ndow$$';
#-------------------------------------

# The game...
hostname=`hostname | awk '{ print tolower($0)}'`;

if [ $# -eq 1 ]; then
	if [ $1 == "-h" ]; then
		dohelp;
		exit 0;
	fi
	if [ $1 == "-l" ]; then
		dolist;
		exit 0;
	fi
fi
# Do we have the dns.txt file?
if [ ! -f ./dns.txt ]; then
	echo "-> ERROR: Could not find dns.txt file.";
	exit 3;
fi

# For the other options we need the IP for the Samba 4 Server...
if [ $# -ne 2 ]; then
        dohelp;
        exit 1;
else
	if [ $1 == "-h" ] || [ $1 == "-l" ]; then
		echo "-> ERROR: -l and -h options do not need any other argument.";
		exit 1;
	fi
	if [ $2 == "-h" ] || [ $2 == "-l" ]; then
		echo "-> ERROR: -l and -h options do not need any other argument.";
		exit 1;
	fi
fi

case "$1" in
        -c) docheck "$2"
        ;;
        -f) dofix "$2"
        ;;
        *) dohelp; exit 1
        ;;
esac
exit 0;
